﻿using System;
using System.Reflection;
using System.Linq;
using System.Linq.Expressions;

namespace Infrastructure.Common
{
    /// <summary>
    /// Business service .
    /// </summary>
    public class BusinessService : IBusinessService
    {
        /// <summary>
        /// Send service data interface .
        /// </summary>
        private ISendServiceData sendService;

        /// <summary>
        /// Mark <see cref="Infrastructure.Common.ISendServiceData"/> instance.
        /// </summary>
        /// <param name="sendService"></param>
        public BusinessService(ISendServiceData sendService)
        {
            this.sendService = sendService;
        }

        /// <summary>
        /// Send business data to service interface.
        /// </summary>
        /// <param name="callAuthor">Object author.</param>
        /// <param name="methodName">Method name.</param>
        /// <param name="parameterInstance">Method call parameter.</param>
        public void SendBusinessData(object callAuthor, string methodName, object parameterInstance)
        {
            object result =
                callAuthor.GetType().GetMethod(methodName).Invoke(callAuthor, new object[] { parameterInstance });
            if (result != null)
            {
                sendService.Send(result);
            }
        }

        /// <summary>
        /// Send business data to service interface.
        /// </summary>
        /// <typeparam name="P"></typeparam>
        /// <param name="callFun"></param>
        /// <param name="parameter"></param>
        public void SendBusinessData<P>(Func<P, object> callFun, P parameter)
        {
            object result = callFun(parameter);
            if (result != null)
            {
                sendService.Send(result);
            }
        }
    }
}
